'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = require('fs');
var path = require('path');
var os = require('os');
var destr = _interopDefault(require('destr'));
var flat = _interopDefault(require('flat'));

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _createForOfIteratorHelper(o) {
  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (o = _unsupportedIterableToArray(o))) {
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var it,
      normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = o[Symbol.iterator]();
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

var RE_KEY_VAL = /^\s*([\w.-]+)\s*=\s*(.*)?\s*$/;
var RE_LINES = /\n|\r|\r\n/;
var defaults = {
  name: '.conf',
  dir: process.cwd(),
  unflatten: true
};

function withDefaults(options) {
  return Object.assign(Object.assign({}, defaults), options);
}
/**
 * Parse rc contents
 */


function parse(contents) {
  var unflatten = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var config = {};
  var lines = contents.split(RE_LINES);

  var _iterator = _createForOfIteratorHelper(lines),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var line = _step.value;
      var match = line.match(RE_KEY_VAL);

      if (!match) {
        continue;
      } // Key


      var key = match[1];

      if (!key || key === '__proto__' || key === 'constructor') {
        continue;
      }

      config[key] = destr(match[2].trim()
      /* val */
      );
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return unflatten ? flat.unflatten(config) : config;
}
/**
 * Parse rc file
 */

function parseFile(path) {
  var unflatten = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

  if (!fs.existsSync(path)) {
    return {};
  }

  return parse(fs.readFileSync(path, 'utf-8'), unflatten);
}
/**
 * Read rc file
 * @param name Name of rc file (default: '.conf')
 * @param dir Working directory (default: process.cwd())
 */

function read(options) {
  options = withDefaults(options);
  return parseFile(path.resolve(options.dir, options.name), options.unflatten);
}
/**
 * Read rc from user directory
 * @param name Name of rc file (default: '.conf')
 */

function readUser(options) {
  options = withDefaults(options);
  options.dir = os.homedir();
  return read(options);
}
/**
 * Serialize rc config
 * @param config Unflatten config
 */

function serialize(config) {
  var unflatten = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  return Object.entries(unflatten ? flat.flatten(config) : config).map(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        val = _ref2[1];

    return "".concat(key, "=").concat(typeof val === 'string' ? val : JSON.stringify(val));
  }).join('\n');
}
/**
 * Write rc config
 * @param config Unflatten config
 * @param name Name of rc file (default: '.conf')
 * @param dir Working directory (default: process.cwd())
 */

function write(config, options) {
  options = withDefaults(options);
  fs.writeFileSync(path.resolve(options.dir, options.name), serialize(config, options.unflatten));
}
/**
 * Write rc from to user directory
 * @param name Name of rc file (default: '.conf')
 */

function writeUser(config, options) {
  options = withDefaults(options);
  options.dir = os.homedir();
  write(config, options);
}

exports.defaults = defaults;
exports.parse = parse;
exports.parseFile = parseFile;
exports.read = read;
exports.readUser = readUser;
exports.serialize = serialize;
exports.write = write;
exports.writeUser = writeUser;
